/* 3x PWM auf Atmega8-PDIP mit 8MHz internem Takt
    Compiliert mit AVR Studio 4.14 und AVR-GCC


		RGB-Moodlight fr ACULED RGB-LED
		im Moment realisierte Modi:
			1: sanfter Farbwechsel
			2: harter Farbwechsel
			3: Farbwahl ber Poti's

		Modi in Planung:
			1: DMX-Ansteuerung
			2: UART-Ansteuerung
			3: IR-Ansteuerung (habe noch eine Beleuchtungs-FB rumzuliegen...)

		
		Moduswechel im Moment nur nach RESET...


Bemerkung des Authors:
	Da dies ein vom Anfnger erstellter Sourcecode ist, bin ich mir sicher,
	da noch umfangreiche nderungen am fertiggestellten Code vonnten sein
	werden. Die noch zu realisierenden Modi sind nicht gerade einfach...
*/

#define RAND_MAX 0xFF
#define F_CPU 8000000UL

#include <stdlib.h>
#include <avr/io.h>        
#include <util/delay.h>

uint8_t mux, result_ADC;
uint16_t rt, gn, bl, rt_neu, gn_neu, bl_neu, result_tmp, t, g, i;

void init_Device (void)
{
	DDRB  =0b00001110;        						//PB1, PB2, PB3 als Ausgang (PWM OCRxx) 
	PORTB =0b11001110;        						//PullUp-Widerstand an PB1, PB2, PB3, PB6, PB7 ein 
  	PORTC =0b00000000;								//PortC als Eingang
	DDRC  =0b00000000;								//PullUp-Widerstnde an PinC aus
	PORTD =0b11111111;								//PortD als Ausgang
	DDRD  =0b00000000;								//PullUp-Widerstnde an PortC aus
	
  	TCCR0 =0b00000001;								//Einstellung der Timer fr 3 Kanal - 8Bit-PWM
	TCCR1A=0b10100001; 
  	TCCR1B=0b00000001; 
  	TCCR2 =0b01100001;
}

void long_delay(uint16_t ms)						// Langzeit Vergerung
{
	for (; ms>0; ms--)
	{
		_delay_ms(1);
	}
}

//Analog-Digital Wandlung (weitestgehend bernommen vom microcontrontroller.net Forum)
void Poti (uint8_t mux)								//Modus3: Farbwahl mittels Poti's
{
	ADCSRA |= (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1);
	ADMUX 	= mux;
	ADMUX  |= (1<<REFS0);							//AVcc als Refenzspannung nutzen und Kanal whlen

	ADCSRA |= (1<<ADSC);							//empfohlender Dummy-Readout zum Warmlaufen des ADC
	while (ADCSRA &(1<<ADSC))						//auf Abschlu der ADC-Wandlung warten
	{}
	result_tmp = ADC;								//Dummy-Readout des ADC notwendig, sonst keine bernahme des nchsten Wertes!!!
	result_tmp = 0;

	ADCSRA |= (1<<ADSC);							//eigentliche Messung
	while (ADCSRA &(1<<ADSC))
	{}
	result_tmp = ADC;								//Abfrage von ADC notwendig, sonst keine bernahme des nchsten Wertes!!!
	result_ADC = (result_tmp >> 2);					//verschieben des Ergebnises um 2 Bits, da nur 8Bit-PWM auf OC2 mglich.
}

void Geschwindigkeit (void)							//Geschwindigkeitssteuerung des Farbwechsels
{
	mux = 0;
	Poti (mux);
	t = (result_tmp * g);
	long_delay (t);
}

void Geschwindigkeit2 (void)						//Geschwindigkeitssteuerung des Farbbergangs
{
	mux = 1;
	Poti (mux);
	t = (result_tmp * g);
	long_delay (t);
}

void Zufallsfarbe (void)							//Farbwahl ber Zufallsfunktion mit Einschrnkung auf 8Bit
{
	rt_neu = rand() % 255;
	gn_neu = rand() % 255;
	bl_neu = rand() % 255;
}

void Farbe_Soft (void)								//Modus1: Weicher Farbbergang
{
	Farbe:
		g = 1;
		while (rt_neu > rt)							//ROT
		{
			rt++;
			OCR1AL = rt;
			Geschwindigkeit2();
		}
		while (rt_neu < rt)
		{
			rt--;
			OCR1AL = rt;
			Geschwindigkeit2();
		}
		Geschwindigkeit();

		while (gn_neu > gn)							//GRN
		{
			gn++;
			OCR1BL = gn;
			Geschwindigkeit2();
		}
		while (gn_neu < gn)
		{
			gn--;
			OCR1BL = gn;
			Geschwindigkeit2();
		}
		Geschwindigkeit();

		while (bl_neu > bl)							//BLAU
		{
			bl++;
			OCR2=bl;
			Geschwindigkeit2();
		}
		while (bl_neu < bl)
		{
			bl--;
			OCR2=bl;
			Geschwindigkeit2();
		}
		Geschwindigkeit();
		if ((rt_neu == rt) && (gn_neu == gn) && (bl_neu == bl))
		{
			Zufallsfarbe();
		}
	goto Farbe;
}

void Farbe_Hart (void)								//Modus2: Harter Farbwechsel
{
	OCR1AL = rt_neu;
	OCR1BL = gn_neu;
	OCR2   = bl_neu;
}

int main (void) 
{
	init_Device();									//Timer und orts initialisieren
	srand (0);										//Zufallszhler initialisieren
	
	i=0;
	i++;
	Modi:
	if (!(PIND & (1<<PIND5)))						//Zufallsmodus - Softer Farbwechsel
	{
		SOFT:

		Zufallsfarbe ();
		Farbe_Soft ();
		i=0;
		goto SOFT;
	}
	if (!(PIND & (1<<PIND6)))						//Zufallsmodus - Harter Farbwechsel
	{
		HART:

		Zufallsfarbe();
		Farbe_Hart();
		g = 5;
		Geschwindigkeit();
		i=0;
		goto HART;
	}
	if (!(PIND & (1<<PIND7)))						//Poti-Modus
	{
		POTI:

		mux = 0;
		Poti (mux);
		rt_neu = result_ADC;
		mux = 1;
		Poti (mux);
		gn_neu = result_ADC;
		mux = 2;
		Poti (mux);
		bl_neu = result_ADC;
		Farbe_Hart ();
		i=0;
		goto POTI;
	}
	if (i==1000)
	{
		goto SOFT;
	}
	_delay_ms(100);
	goto Modi;
}
